/////////////////////////////////////////////////////////////////////////////
// Name:        galaxql.cpp
// Author:      Jari Komppa
// Licence:     
//
// Copyright (c) 2005, Jari Komppa
// All rights reserved.
// 
// Redistribution and use in source and binary forms, 
// with or without modification, are permitted provided 
// that the following conditions are met:
//
// - Redistributions of source code must retain the above 
//   copyright notice, this list of conditions and the 
//   following disclaimer.
// - Redistributions in binary form must reproduce the above 
//   copyright notice, this list of conditions and the 
//   following disclaimer in the documentation and/or other 
//   materials provided with the distribution.
// - The authors' names may not be used to endorse or promote 
//   products derived from this software without specific 
//   prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND 
// CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, 
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
// BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
// TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND 
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
// POSSIBILITY OF SUCH DAMAGE.
//
/////////////////////////////////////////////////////////////////////////////


#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation "galaxql.h"
#endif

#define MAX_STARS (32*1024)

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/glcanvas.h"
#include "wx/splitter.h"
#include "wx/button.h"
#include "wx/textctrl.h"
#include "wx/slider.h"
#include "wx/dcbuffer.h"
#include "wx/panel.h"
#include "wx/image.h"
#include "wx/dialog.h"
#include "wx/checkbox.h"
#if defined(__WXMAC__) || defined(__WXCOCOA__)
#include <OpenGL/glu.h>
#include <sys/syslimits.h>
#else
#include <GL/glu.h>
#endif
#include "sqlite/sqlite3.h"
#include "RegenerationDialog.h"
#include "wx/fs_zip.h"
#include "wx/filesys.h"
#include "wx/filedlg.h"
////@begin includes
////@end includes

#include "galaxql.h"

wxStopWatch ticker;

#define TICK ticker.Time()

#ifndef __WXMSW__
#define stricmp strcasecmp
#endif

const int starcolors[14]= // Hagen numbers
{
    0xD1D1FF, // -3
    0xDBF4FF, // -2
    0xEAFBFF, // -1
    0xFFFFFF, // 0
    0xFFFFE5, // 1
    0xFFFFDB, // 2
    0xFFFFD1, // 3
    0xFFF1DB, // 4
    0xFBE8CF, // 5
    0xFFE6D6, // 6
    0xFAD8D0, // 7
    0xFFD6D6, // 8
    0xFCCECF, // 9
    0xFFD1D1  // 10
};

const int planetcolors[10]= // more or less random numbers
{
    0x7FFF7F, 
    0x3F3F3F, 
    0xFF7FFF, 
    0x7F7F7F, 
    0x3F3FFF, 
    0xFF3F3F, 
    0x3F7FFF, 
    0xFF7F3F, 
    0x7F3FFF, 
    0xFF3F7F  
};


////@begin XPM images
////@end XPM images

/*!
 * Application instance implementation
 */

////@begin implement app
IMPLEMENT_APP( GalaxqlApp )
////@end implement app

/*!
 * GalaxqlApp type definition
 */

IMPLEMENT_CLASS( GalaxqlApp, wxApp )

/*!
 * GalaxqlApp event table definition
 */

BEGIN_EVENT_TABLE( GalaxqlApp, wxApp )

////@begin GalaxqlApp event table entries
////@end GalaxqlApp event table entries

END_EVENT_TABLE()

/*!
 * Constructor for GalaxqlApp
 */

GalaxqlApp::GalaxqlApp()
{
////@begin GalaxqlApp member initialisation
////@end GalaxqlApp member initialisation    
}

/*!
 * Initialisation for GalaxqlApp
 */

bool GalaxqlApp::OnInit()
{    
////@begin GalaxqlApp initialisation
    // Remove the comment markers above and below this block
    // to make permanent changes to the code.

#if wxUSE_XPM
    wxImage::AddHandler( new wxXPMHandler );
#endif
#if wxUSE_LIBPNG
    wxImage::AddHandler( new wxPNGHandler );
#endif
#if wxUSE_LIBJPEG
    wxImage::AddHandler( new wxJPEGHandler );
#endif
#if wxUSE_GIF
    wxImage::AddHandler( new wxGIFHandler );
#endif
    Galaxql* mainWindow = new Galaxql( NULL, ID_FRAME );
    mainWindow->Show(true);
////@end GalaxqlApp initialisation

    return true;
}

/*!
 * Cleanup for GalaxqlApp
 */
int GalaxqlApp::OnExit()
{    
    sqlite3_close(db);
////@begin GalaxqlApp cleanup
    return wxApp::OnExit();
////@end GalaxqlApp cleanup
}


/*!
 * Galaxql type definition
 */

IMPLEMENT_CLASS( Galaxql, wxFrame )

/*!
 * Galaxql event table definition
 */

BEGIN_EVENT_TABLE( Galaxql, wxFrame )

////@begin Galaxql event table entries
    EVT_MENU( ID_REGENERATE25K, Galaxql::OnRegenerate25kClick )

    EVT_MENU( ID_REGENERATE10K, Galaxql::OnRegenerate10kClick )

    EVT_MENU( ID_REGENERATE1KSPHERE, Galaxql::OnRegenerate1ksphereClick )

    EVT_MENU( ID_MENUSAVEQUERY, Galaxql::OnMenusavequeryClick )

    EVT_MENU( ID_MENULOADQUERY, Galaxql::OnMenuloadqueryClick )

    EVT_MENU( ID_QUITAPP, Galaxql::OnQuitappClick )

    EVT_MENU( ID_ABOUTSQL, Galaxql::OnAboutsqlClick )

    EVT_MENU( ID_SQLGETINFO, Galaxql::OnSqlgetinfoClick )

    EVT_MENU( ID_SQLCOUNTSTARS, Galaxql::OnSqlcountstarsClick )

    EVT_MENU( ID_PLANETCOUNT, Galaxql::OnPlanetcountClick )

    EVT_MENU( ID_MAXPLANETS, Galaxql::OnMaxplanetsClick )

    EVT_MENU( ID_SQLRESETHILITE, Galaxql::OnSqlresethiliteClick )

    EVT_MENU( ID_HILIGHT_MOON_PLANETS, Galaxql::OnHilightMoonPlanetsClick )

    EVT_MENU( ID_SQLSHRINK, Galaxql::OnSqlshrinkClick )

    EVT_MENU( ID_TURNGALAXY, Galaxql::OnTurngalaxyClick )

    EVT_MENU( ID_MENURUNQUERY, Galaxql::OnMenurunqueryClick )

    EVT_MENU( ID_MENUGLOW, Galaxql::OnMenuglowClick )

    EVT_MENU( ID_ABOUT, Galaxql::OnAboutClick )

    EVT_NOTEBOOK_PAGE_CHANGED( ID_NOTEBOOK, Galaxql::OnNotebookPageChanged )

    EVT_CHOICE( ID_CHAPTERSELECT, Galaxql::OnChapterselectSelected )

    EVT_BUTTON( ID_GURU_DONE, Galaxql::OnGuruDoneClick )

    EVT_BUTTON( ID_LOADQUERY, Galaxql::OnLoadqueryClick )

    EVT_BUTTON( ID_SAVEQUERY, Galaxql::OnSavequeryClick )

    EVT_BUTTON( ID_RUNQUERY, Galaxql::OnRunqueryClick )

////@end Galaxql event table entries

END_EVENT_TABLE()

/*!
 * Galaxql constructors
 */

Galaxql::Galaxql( )
{
}

Galaxql::Galaxql( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Create( parent, id, caption, pos, size, style );
}

/*!
 * Galaxql creator
 */

bool Galaxql::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
#if defined(__WXMAC__) || defined(__WXCOCOA__)
  // Change working directory to bundle's 'Resources'

  // Get the URL to the application bundle
  CFURLRef appURL = CFBundleCopyBundleURL(CFBundleGetMainBundle());
  
  // Get the directory containing the app
  CFURLRef parentURL = CFURLCreateCopyDeletingLastPathComponent(kCFAllocatorDefault, appURL);
  
  // Convert to native file system representation
  UInt8 path[PATH_MAX];
  CFURLGetFileSystemRepresentation(parentURL, TRUE, path, PATH_MAX);
  
  // Change directory
  chdir((const char*)path);
  chdir("../Resources");
  
  // Clean up
  CFRelease(appURL);
  CFRelease(parentURL);  
#endif

    wxFileSystem::AddHandler(new wxZipFSHandler);
    wxInitAllImageHandlers();

////@begin Galaxql member initialisation
    mGfxPanel = NULL;
    mNotebook = NULL;
    mGuruPicture = NULL;
    mChapterSelect = NULL;
    mGuruSpeaks = NULL;
    mGuruDone = NULL;
    mQueryResult = NULL;
    mQuery = NULL;
    mLoadQuery = NULL;
    mSaveQuery = NULL;
    mRunQuery = NULL;
    mHtmlPanel = NULL;
////@end Galaxql member initialisation

////@begin Galaxql creation
    wxFrame::Create( parent, id, caption, pos, size, style );

    CreateControls();
    SetIcon(GetIconResource(wxT("galaxql.ico")));
////@end Galaxql creation

    wxFileSystem *fs = new wxFileSystem();
    wxFSFile * f = fs->OpenFile("file:galaxql.dat#zip:sad.png");

    mGuruSad.LoadFile(*f->GetStream());
    delete f;
    f = fs->OpenFile("file:galaxql.dat#zip:happy.png");
    mGuruHappy.LoadFile(*f->GetStream());
    delete f;
    f = fs->OpenFile("file:galaxql.dat#zip:normal.png");
    mGuruNormal.LoadFile(*f->GetStream());
    delete f;
    f = fs->OpenFile("file:galaxql.dat#zip:baddb.png");
    mGuruBadDb.LoadFile(*f->GetStream());
    delete f;
    f = fs->OpenFile("file:galaxql.dat#zip:easter.png");
    mGuruEaster.LoadFile(*f->GetStream());
    delete f;
        
    mGuruPicture->SetBitmap(mGuruNormal);
    delete fs;

    mChapterSelect->Append("1. Welcome..");
    mChapterSelect->Append("2. SELECT ..");
    mChapterSelect->Append("3. SELECT .. FROM ..");
    mChapterSelect->Append("4. SELECT .. FROM .. WHERE .. ORDER BY .. DESC");
    mChapterSelect->Append("5. MAX(), MIN(), COUNT(), AVG(), SUM(), NULL");
    mChapterSelect->Append("6. INSERT INTO .. VALUES ..");
    mChapterSelect->Append("7. INSERT INTO .. SELECT ..");
    mChapterSelect->Append("8. Transactions, DELETE FROM .. WHERE ..");
    mChapterSelect->Append("9. UPDATE .. SET .. WHERE ..");
    mChapterSelect->Append("10. SELECT FROM table1, table2.., DISTINCT");
    mChapterSelect->Append("11. SELECT .. FROM (SELECT .. FROM ..)");
    mChapterSelect->Append("12. SELECT FROM .. JOIN ..");
    mChapterSelect->Append("13. CREATE VIEW, DROP VIEW");
    mChapterSelect->Append("14. CREATE TABLE, DROP TABLE");
    mChapterSelect->Append("15. Constraints");
    mChapterSelect->Append("16. ALTER TABLE");
    mChapterSelect->Append("17. SELECT .. GROUP BY .. HAVING ..");
    mChapterSelect->Append("18. UNION, UNION ALL, INTERSECT, EXCEPT");
    mChapterSelect->Append("19. Triggers");
    mChapterSelect->Append("20. Indexes");
    mChapterSelect->Append("That's all for now");
    mChapterSelect->Select(0);
    wxCommandEvent e;
    OnChapterselectSelected(e);

    mHtmlPanel->LoadPage("file:galaxql.dat#zip:index.htm");

    GalaxqlApp * app = (GalaxqlApp*)wxTheApp;  

    int rc = sqlite3_open("galaxql.db",&app->db);
    
    if (rc)
    {
        wxMessageBox(wxT("Can't open galaxql.db!"), wxT("Error"),
                 wxOK | wxICON_ERROR, this);
        sqlite3_close(app->db);
        exit(-1);
    }

    mUserQuery.mRows = 0;
    mUserQuery.mColumns = 0;
    mUserQuery.mStatus = "";


    return TRUE;
}

/*!
 * Control creation for Galaxql
 */

void Galaxql::CreateControls()
{    
////@begin Galaxql content construction
    // Generated by DialogBlocks, 12/04/05 22:08:22 (Personal Edition)

    Galaxql* itemFrame1 = this;

    wxMenuBar* menuBar = new wxMenuBar;
    wxMenu* itemMenu3 = new wxMenu;
    wxMenu* itemMenu4 = new wxMenu;
    itemMenu4->Append(ID_REGENERATE25K, _("Regenerate database (25k)"), _T(""), wxITEM_NORMAL);
    itemMenu4->Append(ID_REGENERATE10K, _("Regenerate database (10k)"), _T(""), wxITEM_NORMAL);
    itemMenu4->Append(ID_REGENERATE1KSPHERE, _("Regenerate database (1k sphere)"), _T(""), wxITEM_NORMAL);
    itemMenu3->Append(ID_RADOPER, _("Radical operations"), itemMenu4);
    itemMenu3->AppendSeparator();
    itemMenu3->Append(ID_MENUSAVEQUERY, _("&Save query.."), _T(""), wxITEM_NORMAL);
    itemMenu3->Append(ID_MENULOADQUERY, _("&Load query.."), _T(""), wxITEM_NORMAL);
    itemMenu3->AppendSeparator();
    itemMenu3->Append(ID_QUITAPP, _("&Quit"), _T(""), wxITEM_NORMAL);
    menuBar->Append(itemMenu3, _("&File"));
    wxMenu* itemMenu13 = new wxMenu;
    itemMenu13->Append(ID_ABOUTSQL, _("About SQL.."), _T(""), wxITEM_NORMAL);
    itemMenu13->AppendSeparator();
    wxMenu* itemMenu16 = new wxMenu;
    itemMenu16->Append(ID_SQLGETINFO, _("Get info about tables"), _T(""), wxITEM_NORMAL);
    itemMenu16->Append(ID_SQLCOUNTSTARS, _("Count stars"), _T(""), wxITEM_NORMAL);
    itemMenu16->Append(ID_PLANETCOUNT, _("Count of planets per star"), _T(""), wxITEM_NORMAL);
    itemMenu16->Append(ID_MAXPLANETS, _("Highest number of planets for a star"), _T(""), wxITEM_NORMAL);
    itemMenu13->Append(ID_MENU, _("Information"), itemMenu16);
    wxMenu* itemMenu21 = new wxMenu;
    itemMenu21->Append(ID_SQLRESETHILITE, _("Reset hilighted state"), _T(""), wxITEM_NORMAL);
    itemMenu21->Append(ID_HILIGHT_MOON_PLANETS, _("Hilight all stars with planets that have moons"), _T(""), wxITEM_NORMAL);
    itemMenu13->Append(ID_MENU1, _("Hilights"), itemMenu21);
    wxMenu* itemMenu24 = new wxMenu;
    itemMenu24->Append(ID_SQLSHRINK, _("Shrink the galaxy"), _T(""), wxITEM_NORMAL);
    itemMenu24->Append(ID_TURNGALAXY, _("Turn galaxy"), _T(""), wxITEM_NORMAL);
    itemMenu13->Append(ID_MENU2, _("Alter geometry"), itemMenu24);
    itemMenu13->AppendSeparator();
    itemMenu13->Append(ID_MENURUNQUERY, _("&Run query\tF5"), _T(""), wxITEM_NORMAL);
    menuBar->Append(itemMenu13, _("SQL"));
    wxMenu* itemMenu29 = new wxMenu;
    itemMenu29->Append(ID_MENUGLOW, _("Render with glow"), _T(""), wxITEM_CHECK);
    menuBar->Append(itemMenu29, _("Graphics"));
    wxMenu* itemMenu31 = new wxMenu;
    itemMenu31->Append(ID_ABOUT, _("About.."), _T(""), wxITEM_NORMAL);
    menuBar->Append(itemMenu31, _("&Help"));
    itemFrame1->SetMenuBar(menuBar);

    wxSplitterWindow* itemSplitterWindow33 = new wxSplitterWindow( itemFrame1, ID_SPLITTERWINDOW, wxDefaultPosition, wxSize(600, 100), wxSP_3DSASH|wxSP_LIVE_UPDATE|wxNO_BORDER );

    mGfxPanel = new GfxPanel( itemSplitterWindow33, ID_PANEL, wxDefaultPosition, wxDefaultSize, wxSUNKEN_BORDER|wxTAB_TRAVERSAL );

    mNotebook = new wxNotebook( itemSplitterWindow33, ID_NOTEBOOK, wxDefaultPosition, wxDefaultSize, wxNB_TOP );

    wxPanel* itemPanel36 = new wxPanel( mNotebook, ID_GURUPANEL, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL );
    wxBoxSizer* itemBoxSizer37 = new wxBoxSizer(wxVERTICAL);
    itemPanel36->SetSizer(itemBoxSizer37);

    wxBoxSizer* itemBoxSizer38 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer37->Add(itemBoxSizer38, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
    wxBitmap mGuruPictureBitmap(wxNullBitmap);
    mGuruPicture = new wxStaticBitmap( itemPanel36, wxID_STATIC, mGuruPictureBitmap, wxDefaultPosition, wxSize(180, 180), wxSTATIC_BORDER );
    itemBoxSizer38->Add(mGuruPicture, 0, wxALIGN_CENTER_VERTICAL|wxALL|wxSHAPED, 5);

    wxBoxSizer* itemBoxSizer40 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer37->Add(itemBoxSizer40, 0, wxGROW|wxALL, 5);
    wxString* mChapterSelectStrings = NULL;
    mChapterSelect = new wxChoice( itemPanel36, ID_CHAPTERSELECT, wxDefaultPosition, wxDefaultSize, 0, mChapterSelectStrings, 0 );
    itemBoxSizer40->Add(mChapterSelect, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer42 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer37->Add(itemBoxSizer42, 4, wxGROW|wxALL, 1);
    mGuruSpeaks = new wxGuruSpeaksPanel( itemPanel36, ID_GURUSPEAKS, wxDefaultPosition, wxDefaultSize, wxSUNKEN_BORDER|wxTAB_TRAVERSAL );
    itemBoxSizer42->Add(mGuruSpeaks, 1, wxGROW|wxALL, 1);

    wxBoxSizer* itemBoxSizer44 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer37->Add(itemBoxSizer44, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
    mGuruDone = new wxButton( itemPanel36, ID_GURU_DONE, _("Ok, I'm done.."), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer44->Add(mGuruDone, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    mNotebook->AddPage(itemPanel36, _("Guru"));

    wxPanel* itemPanel46 = new wxPanel( mNotebook, ID_QUERYPANEL, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL );
    wxBoxSizer* itemBoxSizer47 = new wxBoxSizer(wxHORIZONTAL);
    itemPanel46->SetSizer(itemBoxSizer47);

    wxBoxSizer* itemBoxSizer48 = new wxBoxSizer(wxVERTICAL);
    itemBoxSizer47->Add(itemBoxSizer48, 1, wxGROW|wxALL|wxADJUST_MINSIZE, 1);
    wxStaticText* itemStaticText49 = new wxStaticText( itemPanel46, wxID_STATIC, _("Query Result"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer48->Add(itemStaticText49, 0, wxALIGN_CENTER_HORIZONTAL|wxALL|wxADJUST_MINSIZE, 5);

    mQueryResult = new wxHtmlWindow( itemPanel46, ID_QUERYRESULT, wxDefaultPosition, wxSize(200, 150), wxHW_SCROLLBAR_AUTO|wxSTATIC_BORDER|wxHSCROLL|wxVSCROLL );
    mQueryResult->SetBackgroundColour(wxColour(192, 192, 192));
    itemBoxSizer48->Add(mQueryResult, 1, wxGROW|wxALL, 1);

    wxStaticText* itemStaticText51 = new wxStaticText( itemPanel46, wxID_STATIC, _("Query Editor"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer48->Add(itemStaticText51, 0, wxALIGN_CENTER_HORIZONTAL|wxALL|wxADJUST_MINSIZE, 5);

    mQuery = new wxTextCtrl( itemPanel46, ID_QUERYEDIT, _T(""), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE );
    itemBoxSizer48->Add(mQuery, 1, wxGROW|wxALL|wxADJUST_MINSIZE, 1);

    wxBoxSizer* itemBoxSizer53 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer48->Add(itemBoxSizer53, 0, wxALIGN_RIGHT|wxALL, 5);
    mLoadQuery = new wxButton( itemPanel46, ID_LOADQUERY, _("Load.."), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer53->Add(mLoadQuery, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    mSaveQuery = new wxButton( itemPanel46, ID_SAVEQUERY, _("Save.."), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer53->Add(mSaveQuery, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    mRunQuery = new wxButton( itemPanel46, ID_RUNQUERY, _("Run Query"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer53->Add(mRunQuery, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    mNotebook->AddPage(itemPanel46, _("SQL Query"));

    mHtmlPanel = new wxHtmlWindow( mNotebook, ID_HTMLWINDOW, wxDefaultPosition, wxSize(200, 150), wxHW_SCROLLBAR_AUTO|wxSTATIC_BORDER|wxHSCROLL|wxVSCROLL );

    mNotebook->AddPage(mHtmlPanel, _("Reference"));

    itemSplitterWindow33->SplitVertically(mGfxPanel, mNotebook, 500);

////@end Galaxql content construction
}

/*!
 * Should we show tooltips?
 */

bool Galaxql::ShowToolTips()
{
    return TRUE;
}

/*!
 * Get bitmap resources
 */

wxBitmap Galaxql::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin Galaxql bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end Galaxql bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon Galaxql::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin Galaxql icon retrieval
    wxUnusedVar(name);
    if (name == wxT("galaxql.ico"))
    {
        wxIcon icon(_T("galaxql.ico"), wxBITMAP_TYPE_ICO);
        return icon;
    }
    return wxNullIcon;
////@end Galaxql icon retrieval
}

/*!
 * GfxPanel type definition
 */

IMPLEMENT_CLASS( GfxPanel, wxGLCanvas )

/*!
 * GfxPanel event table definition
 */

BEGIN_EVENT_TABLE( GfxPanel, wxGLCanvas )
    EVT_PAINT(GfxPanel::onPaint)
    EVT_ERASE_BACKGROUND(GfxPanel::onEraseBackground)
    EVT_IDLE(GfxPanel::onIdle)
    EVT_MOUSE_EVENTS(GfxPanel::onMouseEvent)
END_EVENT_TABLE()

/*!
 * GfxPanel constructors
 */

GfxPanel::GfxPanel(wxWindow *parent, int, wxPoint, wxSize, int) : wxGLCanvas(parent, wxID_ANY),xpos(0),ypos(0),mVtx(0),mGatewayVtx(0),mCol(0),mStars(0),mRenderWithGlow(0) {
}


/*!
 * Should we show tooltips?
 */

bool GfxPanel::ShowToolTips()
{
    return TRUE;
}

#define PI 3.14


void GfxPanel::render(float *vtx, float *col, int count) 
{ 
    glMatrixMode( GL_MODELVIEW );
	glPushMatrix();
    glMatrixMode( GL_PROJECTION );
	glPushMatrix();

		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

	//glClear(GL_DEPTH_BUFFER_BIT );
 
    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity( );

    wxSize s = GetSize();

    {
        float x = xpos;
        float y = ypos;
        float z = -1;
        
        float l = 1.5f / sqrt(x*x+y*y+z*z);
        x *= l;
        y *= l;
        z *= l;

        gluLookAt(x,y,z,0,0,0,1,0,0);
    }
//    glTranslatef( 0.0, -80.0, -200.0 );

    //glRotatef( 30, 1.0f, 0, 0);

    float angle = TICK / 50000.0f;

    glRotatef( -60,0,1,0);
    glRotatef( -20,1,0,0);
    glRotatef( angle * 120, 0, 0, 1.0f);

    //glRotatef( angle*20, 0.5, 0.1,0.3);
    //glRotatef(xpos,0,1,0);
    //glRotatef(ypos,1,0,0);


    glEnableClientState(GL_VERTEX_ARRAY);
    glEnableClientState(GL_COLOR_ARRAY);

    glVertexPointer(3, GL_FLOAT, 0, vtx);
    glColorPointer(4, GL_FLOAT, 0, col);

        glDisable(GL_CULL_FACE);
        glEnable(GL_POINT_SMOOTH);
		glHint(GL_POINT_SMOOTH_HINT, GL_NICEST);
        glEnable(GL_POINT_SIZE);
        glEnable(GL_BLEND);
        glDisable(GL_DITHER);
        
        glDisable(GL_DEPTH_TEST);
        glDepthFunc(GL_LEQUAL);
        
        glBlendFunc(GL_SRC_ALPHA, GL_ONE);
        glEnable(GL_VERTEX_ARRAY);
		glEnable(GL_COLOR_ARRAY);
        if (!mRenderWithGlow)
        {
            glPointSize(2.5);
		    glDrawArrays(GL_POINTS, 0, count);        
        }
        else
        {
            float *tempcolor;
            tempcolor=new float[4*mStars];
            glColorPointer(4, GL_FLOAT, 0, tempcolor);    
            int i;
            for (i = 0; i < mStars*4; i++)
                tempcolor[i] = col[i]*0.6;
            glPointSize(1);
		    glDrawArrays(GL_POINTS, 0, count);      
            for (i = 0; i < mStars*4; i++)
                tempcolor[i] = col[i]*0.5;
            glPointSize(2.5);
		    glDrawArrays(GL_POINTS, 0, count);      
            for (i = 0; i < mStars*4; i++)
                tempcolor[i] = col[i]*0.4;
            glPointSize(5);
		    glDrawArrays(GL_POINTS, 0, count);      
            for (i = 0; i < mStars*4; i++)
                tempcolor[i] = col[i]*0.3;
            glPointSize(10);
		    glDrawArrays(GL_POINTS, 0, count);      
            for (i = 0; i < mStars*4; i++)
                tempcolor[i] = col[i]*0.2;
            glPointSize(20);
		    glDrawArrays(GL_POINTS, 0, count);      

            delete[] tempcolor;
        }

    glDisableClientState(GL_COLOR_ARRAY);

    glDisableClientState(GL_VERTEX_ARRAY);    

    if (mGateways > 0)
    {
        glEnableClientState(GL_VERTEX_ARRAY);
        
        glVertexPointer(3, GL_FLOAT, 0, mGatewayVtx);
        glEnable(GL_LINE_SMOOTH);
		glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
        glLineWidth(2);
        glColor4f(0,0,0.5f,0.25f);
        glDrawArrays(GL_LINES, 0, mGateways*2);        

        glDisableClientState(GL_VERTEX_ARRAY);
    }

    if (mHilighted)
    {
        float test[2*3];
        test[0] = mHilightInfo.x;
        test[1] = mHilightInfo.y;
        test[2] = mHilightInfo.z;
        double ox,oy,oz;
        double proj[16], model[16];
        GLint viewport[4];
        
        glGetDoublev(GL_PROJECTION_MATRIX,&proj[0]);
        glGetDoublev(GL_MODELVIEW_MATRIX,&model[0]);
        glGetIntegerv(GL_VIEWPORT,&viewport[0]);
        gluUnProject(s.GetWidth()*0.5,s.GetHeight()*0.24,0,model,proj,viewport,&ox,&oy,&oz);
        test[3] = ox;
        test[4] = oy;
        test[5] = oz;

        glEnableClientState(GL_VERTEX_ARRAY);
        
        glVertexPointer(3, GL_FLOAT, 0, test);
        glEnable(GL_LINE_SMOOTH);
		glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
        glLineWidth(1.5);
        glColor4f(1.0f,0,0,0.5f);
        glDrawArrays(GL_LINES, 0, 2);        

        glDisableClientState(GL_VERTEX_ARRAY);
    }

    glMatrixMode( GL_PROJECTION );
	glPopMatrix();
    glMatrixMode( GL_MODELVIEW );
	glPopMatrix();
}

void GfxPanel::onMouseEvent(wxMouseEvent &event)
{
    if (event.Dragging())
    {
        wxSize s = GetSize();
        ypos = -(event.GetX() - (s.GetWidth()/2.0f))/(s.GetWidth()/4.0f); 
        xpos = (event.GetY() - (s.GetHeight()/2.0f))/(s.GetHeight()/4.0f);
    }
}

inline void GfxPanel::onEraseBackground(wxEraseEvent &) {}

int sql_callback_count(void*userptr,int columns,char**values, char**names)
{
    GfxPanel * d = (GfxPanel*)userptr;
    d->mStars = atoi(values[0]);
    if (d->mStars > MAX_STARS)
        d->mStars = MAX_STARS;
    return 0;
}

int sql_callback_hilight(void*userptr,int columns,char**values, char**names)
{
    GfxPanel * d = (GfxPanel*)userptr;
    d->mHilighted = atoi(values[0]);
    d->mHilightInfo.starclass = atoi(values[1]);
    d->mHilightInfo.x = atof(values[2]);
    d->mHilightInfo.y = atof(values[3]);
    d->mHilightInfo.z = atof(values[4]);
    if (d->mHilighted > MAX_STARS)
        d->mHilighted = 0;
    return 1;
}

int sql_callback_hilightinfo(void*userptr,int columns,char**values, char**names)
{
    //planets.orbitdistance,planets.color,moons.orbitdistance,moons.color
    GfxPanel * d = (GfxPanel*)userptr;
    float planetdistance = atof(values[0]);
    int planetcolor = atoi(values[1]);
    // new planet?
    if (d->mHilightInfo.planets == 0 || d->mHilightInfo.planet[d->mHilightInfo.planets-1].distance != planetdistance)
    {
        d->mHilightInfo.planet[d->mHilightInfo.planets].distance = planetdistance;
        d->mHilightInfo.planet[d->mHilightInfo.planets].color = planetcolor;
        d->mHilightInfo.planets++;
        if (d->mHilightInfo.planets > 20)
            return 1;
    }
    // do we have a moon?
    if (values[2] != 0)
    {
        float moondistance = atof(values[2]);
        int mooncolor = atoi(values[3]);
        d->mHilightInfo.moon[d->mHilightInfo.planets-1][d->mHilightInfo.moons[d->mHilightInfo.planets-1]].distance = moondistance;
        d->mHilightInfo.moon[d->mHilightInfo.planets-1][d->mHilightInfo.moons[d->mHilightInfo.planets-1]].color = mooncolor;
        d->mHilightInfo.moons[d->mHilightInfo.planets-1]++;
        if (d->mHilightInfo.moons[d->mHilightInfo.planets-1] > 20)
            return 1;
    }
    return 0;
}


int sql_callback_grab(void*userptr,int columns,char**values, char**names)
{
    GfxPanel * d = (GfxPanel*)userptr;
    if (d->mInspectedStar >= MAX_STARS)
        return 1;
    
    d->mVtx[d->mInspectedStar*3+0] = atof(values[0]);
    d->mVtx[d->mInspectedStar*3+1] = atof(values[1]);
    d->mVtx[d->mInspectedStar*3+2] = atof(values[2]);

    int color = atoi(values[3])+3;
    float intensity = atof(values[4]);
    int hilite = values[5]?atoi(values[5]):0;
    d->mCol[d->mInspectedStar*4+0]=(((starcolors[color]>>0) & 0xff) / (float)0xff)*(intensity) + hilite;
    d->mCol[d->mInspectedStar*4+1]=(((starcolors[color]>>8) & 0xff) / (float)0xff)*(intensity) + hilite;
    d->mCol[d->mInspectedStar*4+2]=(((starcolors[color]>>16) & 0xff) / (float)0xff)*(intensity) + hilite;
    d->mCol[d->mInspectedStar*4+3]=1;
    
    d->mInspectedStar++;
    
    return 0;
}

int sql_callback_gateways(void*userptr,int columns,char**values, char**names)
{
    GfxPanel * d = (GfxPanel*)userptr;
    
    d->mGatewayVtx[d->mGateways*6+0] = atof(values[0]);
    d->mGatewayVtx[d->mGateways*6+1] = atof(values[1]);
    d->mGatewayVtx[d->mGateways*6+2] = atof(values[2]);
    d->mGatewayVtx[d->mGateways*6+3] = atof(values[3]);
    d->mGatewayVtx[d->mGateways*6+4] = atof(values[4]);
    d->mGatewayVtx[d->mGateways*6+5] = atof(values[5]);

    d->mGateways++;
    
    return 0;
}

void GfxPanel::render_system()
{
    float angle = (TICK / 5000.0f) + 1337;
    int vtxcount = 1;
    vtxcount += mHilightInfo.planets;
    int i;
    for (i = 0; i < mHilightInfo.planets; i++)
        vtxcount += mHilightInfo.moons[i];
    float *vtx = new float[3*vtxcount];
    float *col = new float[4*vtxcount];

    vtx[0]=vtx[1]=0;
    vtx[2]=-1;
    int color = mHilightInfo.starclass + 3;
    col[0]=(((starcolors[color]>>0) & 0xff) / (float)0xff);//*intensity + hilite;
    col[1]=(((starcolors[color]>>8) & 0xff) / (float)0xff);//*intensity + hilite;
    col[2]=(((starcolors[color]>>16) & 0xff) / (float)0xff);//*intensity + hilite;
    col[3]=1;

    int moonid = 0;
    int moonoffset = mHilightInfo.planets + 1;
    for (i = 0; i < mHilightInfo.planets; i++)
    {
        float d = mHilightInfo.planet[i].distance / 150000;
        vtx[i*3+3+0]=sin(angle/d)*d;
        vtx[i*3+3+1]=cos(angle/d)*d;
        vtx[i*3+3+2]=-1;
        int color = mHilightInfo.planet[i].color / 2;
        if (color < 0 || color > 9) color = 0;
        col[i*4+4+0]=(((planetcolors[color]>>0) & 0xff) / (float)0xff);//*intensity + hilite;
        col[i*4+4+1]=(((planetcolors[color]>>8) & 0xff) / (float)0xff);//*intensity + hilite;
        col[i*4+4+2]=(((planetcolors[color]>>16) & 0xff) / (float)0xff);//*intensity + hilite;
        col[i*4+4+3]=1;
        int j;
        for (j = 0; j < mHilightInfo.moons[i]; j++)
        {
            float md = mHilightInfo.moon[i][j].distance / 150000;
            vtx[moonid*3+moonoffset*3+0] = vtx[i*3+3+0] + sin(angle/md)*md;
            vtx[moonid*3+moonoffset*3+1] = vtx[i*3+3+1] + cos(angle/md)*md;
            vtx[moonid*3+moonoffset*3+2] = -1;           
            int color = mHilightInfo.moon[i][j].color / 2;
            if (color < 0 || color > 9) color = 0;
            col[moonid*4+moonoffset*4+0]=(((planetcolors[color]>>0) & 0xff) / (float)0xff);//*intensity + hilite;
            col[moonid*4+moonoffset*4+1]=(((planetcolors[color]>>8) & 0xff) / (float)0xff);//*intensity + hilite;
            col[moonid*4+moonoffset*4+2]=(((planetcolors[color]>>16) & 0xff) / (float)0xff);//*intensity + hilite;
            col[moonid*4+moonoffset*4+3]=1;
            moonid++;
        }
    }
    glMatrixMode( GL_MODELVIEW );
	glPushMatrix();
    glMatrixMode( GL_PROJECTION );
	glPushMatrix();

    glColor4f((float)(0x24/(float)0xff), 
                (float)(0x3a/(float)0xff),
                (float)(0x49/(float)0xff), 0.5 );
    wxSize s = GetSize();

    glViewport( (int)((s.GetWidth() - s.GetWidth()/1.5f)/2.0f), -s.GetHeight()/10, (int)(s.GetWidth() / 1.5f), (int)(s.GetHeight() / 1.5f));

    {
        float x = -0.75f;//xpos;
        float y = 0;//ypos;
        float z = -1.0f;
        
        float l = 2.5f / sqrt(x*x+y*y+z*z);
        x *= l;
        y *= l;
        z *= l;

        gluLookAt(x,y,z,0,0,-1,1,0,0);
    }

    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
    glBegin(GL_QUADS);
        glVertex3f(-0.7f, 0.7f,-1.1f);
        glVertex3f( 0.7f, 0.7f,-1.1f);
        glVertex3f( 0.7f,-0.7f,-1.1f);
        glVertex3f(-0.7f,-0.7f,-1.1f);
    /*
        glVertex3f(-0.3,-0.175,-1);
        glVertex3f(0.3,-0.175,-1);
        glVertex3f(0.9,-0.75,-1);
        glVertex3f(-0.9,-0.75,-1);
        */
    glEnd();


    glEnableClientState(GL_VERTEX_ARRAY);

    glVertexPointer(3, GL_FLOAT, 0, vtx);
    glColor4f(0,0,0,0.5f);

    glPointSize(7);
    glDrawArrays(GL_POINTS, 0, 1);        
    glPointSize(5);
    glDrawArrays(GL_POINTS, 1, mHilightInfo.planets);        
    glPointSize(2);
    glDrawArrays(GL_POINTS, 1+mHilightInfo.planets, vtxcount - (mHilightInfo.planets+1));        

    glEnableClientState(GL_COLOR_ARRAY);
    glColorPointer(4, GL_FLOAT, 0, col);

    for (i = 0; i < vtxcount; i++)
        vtx[i*3+2]=-1.04f;

    glPointSize(7);
    glDrawArrays(GL_POINTS, 0, 1);        
    glPointSize(5);
    glDrawArrays(GL_POINTS, 1, mHilightInfo.planets);        
    glPointSize(2);
    glDrawArrays(GL_POINTS, 1+mHilightInfo.planets, vtxcount - (mHilightInfo.planets+1));        

    glDisableClientState(GL_COLOR_ARRAY);
    glDisableClientState(GL_VERTEX_ARRAY);

    delete[] col;
    delete[] vtx;

    glMatrixMode( GL_PROJECTION );
	glPopMatrix();
    glMatrixMode( GL_MODELVIEW );
	glPopMatrix();

};

void GfxPanel::onPaint(wxPaintEvent &) {
    wxPaintDC dc(this);
    SetCurrent();

    GalaxqlApp * app = (GalaxqlApp*)wxTheApp;  

    if (mVtx == NULL)
    {
        mStars = 0;
        sqlite3_exec(
                app->db,
                "select count() from gateways",
                sql_callback_count,
                this,
                NULL);
        mGateways = mStars;

        mStars = 0;
        sqlite3_exec(
                app->db,
                "select count() from stars",
                sql_callback_count,
                this,
                NULL);

        delete[] mVtx;
        delete[] mCol;
        delete[] mGatewayVtx;
        mVtx = new float[mStars * 3];
        mCol = new float[mStars * 4];
        mGatewayVtx = new float[mGateways * 3 * 2];
        mInspectedStar = 0;
        sqlite3_exec(
                app->db, // Since NULL*0+1=NULL, light is either NULL or 1.
                "SELECT x,y,z,class,intensity,hilight.starid*0+1 AS light "
                "FROM stars "
                "LEFT JOIN hilight "
                "ON stars.starid=hilight.starid",                
                sql_callback_grab,
                this,
                NULL);

        mHilighted = 0;
        sqlite3_exec(
                app->db,
                "SELECT stars.starid, class, x, y, z "
                "FROM stars,hilight "
                "WHERE stars.starid=hilight.starid",
                sql_callback_hilight,
                this,
                NULL);
        if (mHilighted)
        {
            mHilightInfo.planets = 0;
            int i;
            for (i = 0; i < 20; i++)
                mHilightInfo.moons[i] = 0;
            char temp[256];
            sprintf(temp, "SELECT planets.orbitdistance,planets.color,moons.orbitdistance,moons.color "
                          "FROM planets "
                          "LEFT JOIN moons "
                          "ON planets.starid=%d "
                          "AND planets.planetid=moons.planetid",mHilighted);
            sqlite3_exec(
                    app->db,
                    temp,
                    sql_callback_hilightinfo,
                    this,
                    NULL);
        }
        if (mGateways > 0)
        {
            mGateways = 0;
            sqlite3_exec(
                    app->db,
                    "SELECT st1.x,st1.y,st1.z,st2.x,st2.y,st2.z "
                    "FROM stars AS st1, stars AS st2, gateways "
                    "WHERE st1.starid=gateways.star1 "
                    "AND st2.starid=gateways.star2",
                    sql_callback_gateways,
                    this,
                    NULL);
        }
    }


    wxSize s = GetSize();


    glShadeModel( GL_SMOOTH );

    glCullFace( GL_BACK );
    glFrontFace( GL_CCW );
    glDisable( GL_CULL_FACE );

    glDisable(GL_DEPTH_TEST);
    glDepthFunc(GL_LEQUAL);

    glClearColor( (float)(0x24/(float)0xff*0.25), 
                  (float)(0x3a/(float)0xff*0.25),
                  (float)(0x49/(float)0xff*0.25), 0 );

    glViewport( 0, 0, s.GetWidth(), s.GetHeight());

    glMatrixMode( GL_PROJECTION );
    glLoadIdentity( );
    gluPerspective( 70.0, s.GetWidth()/(float)s.GetHeight(), 0.01, 1024.0 );

    glClear(GL_COLOR_BUFFER_BIT);    

    render(mVtx,mCol,mStars);

    if (mHilighted)
    {
        render_system();
    }

    glFlush(); 
    SwapBuffers();
}

void GfxPanel::onIdle(wxIdleEvent &event) {

    // refresh the panel
    Refresh(false);
    
    // If we don't request more 'onIdle' events, the starmap will just stay idle
    // until the next window message (on some platforms)
    event.RequestMore();

    // throttle to keep from flooding the event queue
    wxMilliSleep(33);
}

/*!
 * Get bitmap resources
 */

wxBitmap GfxPanel::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin GfxPanel bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end GfxPanel bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon GfxPanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin GfxPanel icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end GfxPanel icon retrieval
}

#if 0 // dialogblocks wants to regenerate these all the time..

/*!
 * GfxPanel type definition
 */

IMPLEMENT_DYNAMIC_CLASS( GfxPanel, wxGLCanvas )

/*!
 * GfxPanel event table definition
 */

BEGIN_EVENT_TABLE( GfxPanel, wxGLCanvas )

////@begin GfxPanel event table entries
////@end GfxPanel event table entries

END_EVENT_TABLE()

/*!
 * GfxPanel constructors
 */

GfxPanel::GfxPanel( )
{
}

GfxPanel::GfxPanel( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
    Create(parent, id, pos, size, style);
}

/*!
 * GfxPanel creator
 */

bool GfxPanel::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin GfxPanel member initialisation
////@end GfxPanel member initialisation

////@begin GfxPanel creation
    wxGLCanvas::Create( parent, id, pos, size, style );

    CreateControls();
////@end GfxPanel creation
    return TRUE;
}

/*!
 * Control creation for GfxPanel
 */

void GfxPanel::CreateControls()
{    
////@begin GfxPanel content construction
    // Generated by DialogBlocks, 12/04/05 22:08:22 (Personal Edition)

    GfxPanel* itemGLCanvas34 = this;

////@end GfxPanel content construction
}

/*!
 * Should we show tooltips?
 */

bool GfxPanel::ShowToolTips()
{
    return TRUE;
}

/*!
 * Get bitmap resources
 */

wxBitmap GfxPanel::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin GfxPanel bitmap retrieval
    return wxNullBitmap;
////@end GfxPanel bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon GfxPanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin GfxPanel icon retrieval
    return wxNullIcon;
////@end GfxPanel icon retrieval
}
/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON
 */

#endif 

int sql_callback(void*userptr,int columns,char**values, char**names)
{
    Query * d = (Query*)userptr;

    int i;
    if (d->mRows == 0)
    {
        if (columns > 32)
        {
            d->mStatus = "Dataset too wide. Columns truncated. ";
            columns = 32;
        }
        
        d->mColumns = columns;
        for (i = 0; i < columns; i++)
        {
            d->mResults[d->mRows][i] = names[i];
        }
        d->mRows++;
    }
    if (columns > 32)
    {
        columns = 32;
    }

    for (i = 0; i < columns; i++)
    {
        // TODO: handle NULLs in some other way
        if (values[i]== NULL)
            d->mResults[d->mRows][i] = "<font color=red>NULL</font>";
        else
        {
            d->mResults[d->mRows][i] = values[i];
            // TODO: this should be done when printing the data..
            d->mResults[d->mRows][i].Replace("\n","<br>",true);

        }
    }
    
    d->mRows++;

    if (d->mRows == 1001)
    {
        d->mStatus += "Dataset too large. Results truncated.";
        return 1;
    }
    return 0;
}

int guess_align(const char * data)
{
    const char * walker = data;
    while (*walker)
    {
        if ((*walker < '0' || *walker > '9') && *walker != '-')
            return 1;
        walker++;
    }
    return 0;
}

void Galaxql::OnRunqueryClick( wxCommandEvent& event )
{
    GalaxqlApp * app = (GalaxqlApp*)wxTheApp;  
    mLoadQuery->Enable(0);
    mSaveQuery->Enable(0);
    mRunQuery->Enable(0);
    char *errmsg = NULL;
    mUserQuery.mRows = 0;
    mUserQuery.mColumns = 0;
    mUserQuery.mStatus = "";
    int i,j;
    for (i = 0; i < 1001; i++)
        for (j = 0; j < 32; j++)
            mUserQuery.mResults[i][j] = "";

    int rc =  sqlite3_exec(
                app->db,
                mQuery->GetValue(),
                sql_callback,
                &mUserQuery,
                &errmsg);

    if (rc != SQLITE_OK && rc != SQLITE_ABORT)
    {
        wxString s;
        s = "<html><body bgcolor=#c0c0c0><font color=red>";
        s += errmsg;
        s += "</font></body></html>";
        mQueryResult->SetPage(s);
        sqlite3_free(errmsg);
    }
    else
    {
        if (mUserQuery.mRows < 1)
        {
            mQueryResult->SetPage("<html><body bgcolor=#c0c0c0>Query ok - no rows returned</body></html>");
        }
        else
        {
            // wxwidgets string gets slow when handling long strings,
            // so compiling one row of html in a temporary string is
            // MUCH faster than adding to the master string all the time.
            wxString s,s2;
            s+="<html><body bgcolor=#c0c0c0><table border=0 cellpadding=0 cellspacing=2>";
            int i;
            for (i = 0; i < mUserQuery.mRows; i++)
            {
                s2="";
                if (i == 0)
                    s2 += "<tr bgcolor=#f0f0f0>";
                else
                if ((((i+1) >> 1) & 1) == 0)
                    s2 += "<tr bgcolor=#d0d0d0>";
                else
                    s2 += "<tr bgcolor=#b0b0b0>";
                int j;
                for (j = 0; j < mUserQuery.mColumns; j++)
                {
                    if (i == 0)
                    {
                        s2 += "<td align=center><b>";
                    }
                    else
                    {
                        if (guess_align(mUserQuery.mResults[i][j]))
                            s2 += "<td>";
                        else
                            s2 += "<td align=right>";
                    }
                    s2 += mUserQuery.mResults[i][j];

                    if (i == 0)
                    {
                        s2 += "</b>";
                    }
                    s2 += "</td>";
                }
                s2 += "</tr>";
                s += s2;
            }
            s2 = "";
            s2 += "</table><br clear=all>";
            s2 += mUserQuery.mStatus;
            s2 += "</body></html>";
            s += s2;
            mQueryResult->SetPage(s);
        }
        // refresh starmap. Only really needed when the database changes,
        // but there's no dead-easy way to detect this.        
        delete[] mGfxPanel->mVtx;
        delete[] mGfxPanel->mCol;
        mGfxPanel->mVtx = NULL;
        mGfxPanel->mCol = NULL;
    }

    mLoadQuery->Enable(1);
    mSaveQuery->Enable(1);
    mRunQuery->Enable(1);
    event.Skip();
}


void Galaxql::RegenerateWorld(int which)
{
    RegenerationDialog *reg = new RegenerationDialog(this);
    reg->mProgress->SetValue(0);
    reg->Show(1);
    
    GalaxqlApp * app = (GalaxqlApp*)wxTheApp;  
    sqlite3_close(app->db);
    remove("galaxql.db");
    sqlite3_open("galaxql.db",&app->db);

    sqlite3_exec(
                app->db,
                "CREATE TABLE stars (\n"
                "starid INTEGER PRIMARY KEY, \n"
                "name TEXT, \n"
                "x DOUBLE NOT NULL, \n"
                "y DOUBLE NOT NULL, \n"
                "z DOUBLE NOT NULL, \n"
                "class INTEGER NOT NULL, \n"
                "intensity DOUBLE NOT NULL\n"
                ");",
                NULL,
                NULL,
                NULL);

    sqlite3_exec(
                app->db,
                "CREATE TABLE hilight (\n"
                "starid INTEGER UNIQUE \n"
                ");",
                NULL,
                NULL,
                NULL);

    sqlite3_exec(
                app->db,
                "CREATE TABLE planets (\n"
                "planetid INTEGER PRIMARY KEY, \n"
                "starid INTEGER NOT NULL, \n"
                "orbitdistance DOUBLE NOT NULL, \n"
                "name TEXT, \n"
                "color INTEGER NOT NULL, \n"
                "radius DOUBLE NOT NULL"
                ");",
                NULL,
                NULL,
                NULL);

    sqlite3_exec(
                app->db,
                "CREATE TABLE moons (\n"
                "moonid INTEGER PRIMARY KEY, \n"
                "planetid INTEGER NOT NULL, \n"
                "orbitdistance DOUBLE NOT NULL, \n"
                "name TEXT, \n"
                "color INTEGER NOT NULL, \n"
                "radius DOUBLE NOT NULL\n"
                ");",
                NULL,
                NULL,
                NULL);

    sqlite3_exec(
                app->db,
                "BEGIN",
                NULL,
                NULL,
                NULL);

    switch (which)
    {
    case 1:
        {
            int planetid = 0;
            float step = 1.0f/2.5f;
            srand(0);
            int i;
            for (i = 0; i < 25000; i++)
            {
                float x = sin(pow((float)i*step,0.1f)*10 + (i&3)*3.14*0.5) + (rand() / (float)(RAND_MAX/2)-1)*0.6;
                float y = cos(pow((float)i*step,0.1f)*10 + (i&3)*3.14*0.5) + (rand() / (float)(RAND_MAX/2)-1)*0.6;
                float l = sqrt(x*x+y*y);
                float z = sin((i*step)/1000.0)*((10000-(i*step))/10000.0f) * (rand() / (float)(RAND_MAX/2)-1) * 0.2;
                x = (x/l) * 0.5 * i * 0.00025 * step;
                y = (y/l) * 0.5 * i * 0.00025 * step;
                z = z * 0.75;
                int color = rand()%14;
                float intensity = 2*(rand()%31 + (14-color))/450.0f;
                char temp[1024];
                sprintf(temp,"INSERT INTO stars ("
                        "name,"
                        "x,"
                        "y,"
                        "z,"
                        "class,"
                        "intensity"
                        ") VALUES ("
                        "'%c-%05d',"
                        "%3.9f,"
                        "%3.9f,"
                        "%3.9f,"
                        "%d,"
                        "%3.9f"
                        ");",
                        "OBAFGKM"[color/2],i,
                        x,
                        y,
                        z,
                        color-3,
                        intensity);

                sqlite3_exec(
                        app->db,
                        &temp[0],
                        NULL,
                        NULL,
                        NULL);

                if (rand() % 100 < 20)
                {
                    // 20% of stars have planets
                    int planets = (int)(pow((rand()/(float)(RAND_MAX)),2)*20);
                    int p;
                    for (p = 0; p < planets; p++)
                    {
                        sprintf(temp,"INSERT INTO planets ("
                                "starid, "
                                "orbitdistance, "
                                "name, "
                                "color, "
                                "radius "
                                ") VALUES ("
                                "%d,"
                                "%3.9f,"
                                "'%c-%05d/%c',"
                                "%d,"
                                "%3.9f"
                                ");",
                                i+1,
                                (rand()/(float)(RAND_MAX))*1000+((p+1) / 20.0f) * 100000,
                                "OBAFGKM"[color/2],i+1,"ABCDEFGHIJKLMNOPQRSTUVWXYZ"[p],
                                rand()%16,
                                (rand()/(float)(RAND_MAX))*90+10);

                        sqlite3_exec(
                                app->db,
                                &temp[0],
                                NULL,
                                NULL,
                                NULL);
                        planetid++;


                        if (rand() % 100 < 20)
                        {
                            // 20% of planets have moons
                            int moons = (int)(pow((rand()/(float)(RAND_MAX)),2)*20);
                            int m;
                            for (m = 0; m < moons; m++)
                            {
                                sprintf(temp,"INSERT INTO moons ("
                                        "planetid, "
                                        "orbitdistance, "
                                        "name, "
                                        "color, "
                                        "radius "
                                        ") VALUES ("
                                        "%d,"
                                        "%3.9f,"
                                        "'%c-%05d/%c-%d',"
                                        "%d,"
                                        "%3.9f"
                                        ");",
                                        planetid,
                                        (rand()/(float)(RAND_MAX))*100+((m + 1) / 20.0f) * 10000,
                                        "OBAFGKM"[color/2],i,"ABCDEFGHIJKLMNOPQRSTUVWXYZ"[p],m+1,
                                        rand()%16,
                                        (rand()/(float)(RAND_MAX))*9+1);

                                sqlite3_exec(
                                        app->db,
                                        &temp[0],
                                        NULL,
                                        NULL,
                                        NULL);
                            }
                        }
                    }

                }

                reg->mProgress->SetValue((i * 100)/25000);
            }
        }
        break;
    case 2:
        {
            float step = 1.0f;
            srand(0);
            int i;
            for (i = 0; i < 10000; i++)
            {
                float x = sin(pow((float)i*step,0.1f)*10 + (i&3)*3.14*0.5) + (rand() / (float)(RAND_MAX/2)-1)*0.6;
                float y = cos(pow((float)i*step,0.1f)*10 + (i&3)*3.14*0.5) + (rand() / (float)(RAND_MAX/2)-1)*0.6;
                float l = sqrt(x*x+y*y);
                float z = sin((i*step)/1000.0)*((10000-(i*step))/10000.0f) * (rand() / (float)(RAND_MAX/2)-1) * 0.2;
                x = (x/l) * 0.5 * i * 0.00025 * step;
                y = (y/l) * 0.5 * i * 0.00025 * step;
                z = z * 0.75;
                int color = rand()%14;
                float intensity = 2*(rand()%31 + (14-color))/450.0f;
                char temp[1024];
                sprintf(temp,"INSERT INTO stars ("
                        "name,"
                        "xpos,"
                        "ypos,"
                        "zpos,"
                        "class,"
                        "intensity"
                        ") VALUES ("
                        "'%c-%05d',"
                        "%3.9f,"
                        "%3.9f,"
                        "%3.9f,"
                        "%d,"
                        "%3.9f"
                        ");",
                        "OBAFGKM"[color/2],i,
                        x,
                        y,
                        z,
                        color-3,
                        intensity);

                sqlite3_exec(
                        app->db,
                        &temp[0],
                        NULL,
                        NULL,
                        NULL);
                reg->mProgress->SetValue((i * 100) / 10000);
            }
        }
        break;
    case 3:
        {
            srand(0);
            int i;
            for (i = 0; i < 1000; i++)
            {
                float x = (rand() / (float)(RAND_MAX/2)-1);
                float y = (rand() / (float)(RAND_MAX/2)-1);
                float z = (rand() / (float)(RAND_MAX/2)-1);
                float l = sqrt(x*x+y*y+z*z);
                x/=l;
                y/=l;
                z/=l;
                x*=0.5;
                y*=0.5;
                z*=0.5;

                int color = rand()%14;
                float intensity = 2*(rand()%31 + (14-color))/450.0f;
                char temp[1024];
                sprintf(temp,"INSERT INTO stars ("
                        "name,"
                        "xpos,"
                        "ypos,"
                        "zpos,"
                        "class,"
                        "intensity"
                        ") VALUES ("
                        "'%c-%05d',"
                        "%3.9f,"
                        "%3.9f,"
                        "%3.9f,"
                        "%d,"
                        "%3.9f"
                        ");",
                        "OBAFGKM"[color/2],i,
                        x,
                        y,
                        z,
                        color-3,
                        intensity);

                sqlite3_exec(
                        app->db,
                        &temp[0],
                        NULL,
                        NULL,
                        NULL);

                reg->mProgress->SetValue((i * 100) / 1000);
            }
        }
        break;
    }

    sqlite3_exec(
                app->db,
                "COMMIT",
                NULL,
                NULL,
                NULL);

    sqlite3_exec(
                app->db,
                "CREATE INDEX planets_starid \nON planets (starid)",
                NULL,
                NULL,
                NULL);

    sqlite3_exec(
                app->db,
                "CREATE INDEX moons_planetid \nON moons (planetid)",
                NULL,
                NULL,
                NULL);

    delete[] mGfxPanel->mVtx;
    delete[] mGfxPanel->mCol;
    mGfxPanel->mVtx = NULL;
    mGfxPanel->mCol = NULL;
    reg->Show(0);
    delete reg;
}

/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_MENUITEM
 */

void Galaxql::OnRegenerate25kClick( wxCommandEvent& event )
{
    RegenerateWorld(1);
    event.Skip();
}

/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_MENU
 */

void Galaxql::OnQuitappClick( wxCommandEvent& event )
{
    Close();
    event.Skip();
}

/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_MENUITEM1
 */

void Galaxql::OnAboutClick( wxCommandEvent& event )
{
    wxString msg;
    msg.Printf(wxT(
        "GalaXQL\n"
        "\n"
        "An interactive SQL tutorial\n"
        "\n"
        "Copyright 2005 Jari Komppa\n"
        "\n"
        "http://iki.fi/sol\n"
        "\n"
        "Uses wxWidgets and SQLite\n"
        "http://www.wxwidgets.org\n"
        "http://www.sqlite.org\n"
        "\n"
        "Prof. Guru images by Mikko Finneman"));

    wxMessageBox(msg, wxT("About GalaXQL"),
                 wxOK | wxICON_INFORMATION, this);
    event.Skip();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_ABOUTSQL
 */

void Galaxql::OnAboutsqlClick( wxCommandEvent& event )
{
    wxString msg;
    msg.Printf(wxT("GalaXQL lets you modify the fictional galaxy using SQL statements.\n\n"
                   "The commands in this menu change the SQL query field.\n\n"
                   "(Note that if you manage to destroy the galaxy, you can always\n"
                   "regenerate it under the 'file' menu)"));

    wxMessageBox(msg, wxT("About SQL"),
                 wxOK | wxICON_INFORMATION, this);
    event.Skip();
}

/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_SQLGETINFO
 */

void Galaxql::OnSqlgetinfoClick( wxCommandEvent& event )
{
    mQuery->SetValue("SELECT sql FROM sqlite_master WHERE type='table'");    
    event.Skip();
}

/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_SQLCOUNTSTARS
 */

void Galaxql::OnSqlcountstarsClick( wxCommandEvent& event )
{
    mQuery->SetValue("SELECT COUNT() FROM stars");    
    event.Skip();
}

/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_SQLRESETHILITE
 */

void Galaxql::OnSqlresethiliteClick( wxCommandEvent& event )
{
    mQuery->SetValue("DELETE FROM hilight");    
    event.Skip();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_REGENERATE10K
 */

void Galaxql::OnRegenerate10kClick( wxCommandEvent& event )
{
    RegenerateWorld(2);
    event.Skip();
}

/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_REGENERATE1KSPHERE
 */

void Galaxql::OnRegenerate1ksphereClick( wxCommandEvent& event )
{
    RegenerateWorld(3);
    event.Skip();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_SQLSHRINK
 */

void Galaxql::OnSqlshrinkClick( wxCommandEvent& event )
{
    mQuery->SetValue("UPDATE stars SET \nx=x*0.5, \ny=y*0.5, \nz=z*0.5");    
    event.Skip();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_HILIGHT_MOON_PLANETS
 */

void Galaxql::OnHilightMoonPlanetsClick( wxCommandEvent& event )
{
    mQuery->SetValue(
                "INSERT INTO \n"
                "hilight (starid) \n"
                "SELECT DISTINCT \n"
                "stars.starid FROM \n"
                "stars,planets,moons \n"
                "WHERE \n"
                "moons.planetid=planets.planetid \n"
                "AND planets.starid=stars.starid \n");
    event.Skip();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_TURNGALAXY
 */

void Galaxql::OnTurngalaxyClick( wxCommandEvent& event )
{
    mQuery->SetValue("UPDATE stars SET \nx=z, \nz=x \n");    
    event.Skip();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_PLANETCOUNT
 */

void Galaxql::OnPlanetcountClick( wxCommandEvent& event )
{
    mQuery->SetValue(
        "SELECT stars.starid AS starid, \n"
        "COUNT(planets.planetid) AS planet_count \n"
        "FROM planets, stars \n"
        "WHERE stars.starid=planets.starid \n"
        "GROUP BY stars.starid \n");
    event.Skip();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_MAXPLANETS
 */

void Galaxql::OnMaxplanetsClick( wxCommandEvent& event )
{
    mQuery->SetValue(
        "SELECT MAX(planet_count) from (SELECT stars.starid AS starid, \n"
        "COUNT(planets.planetid) AS planet_count \n"
        "FROM planets, stars \n"
        "WHERE stars.starid=planets.starid \n"
        "GROUP BY stars.starid) \n");
    event.Skip();
}

float Galaxql::min_col(Query &q, int colno)
{
    float val = atof(q.mResults[1][colno]);
    int i;
    for (i = 2; i < q.mRows; i++)
        if (atof(q.mResults[i][colno]) < val)
            val = atof(q.mResults[i][colno]);
    return val;
}

float Galaxql::max_col(Query &q, int colno)
{
    float val = atof(q.mResults[1][colno]);
    int i;
    for (i = 2; i < q.mRows; i++)
        if (atof(q.mResults[i][colno]) > val)
            val = atof(q.mResults[i][colno]);
    return val;
}

int Galaxql::sorted_col(Query &q, int colno)
{
    float v = atof(q.mResults[1][colno]);
    float w = atof(q.mResults[2][colno]);
    int i = 2;
    while (i < q.mRows && v <= w)
    {
        v = w;
        i++;
        w = atof(q.mResults[i][colno]);
    }
    if (i >= q.mRows)
        return 1;
    return 0;
}

int Galaxql::equals_col(Query &q, int colno, int value)
{
    int i;
    for (i = 1; i < q.mRows; i++)
        if (atoi(q.mResults[i][colno]) != value)
            return 0;
    return 1;
}

int Galaxql::check_pairs(Query &q)
{
    int i, j;
    for (j = 0; j < q.mColumns; j++)
    for (i = 0; i < (q.mRows-1)/2; i++)
    {
        if (stricmp(q.mResults[i*2+1][j],q.mResults[i*2+2][j])!=0)
            return 0;
    }
    return 1;
}

int Galaxql::compare_queries(Query &q1, Query &q2)
{
    if (q1.mRows == 0)
        return 0;
    if (q1.mColumns != q2.mColumns)
        return -2;
    if (q1.mRows != q2.mRows)
        return -1;
    int i, j;
    for (i = 0; i < q1.mColumns; i++)
        if (stricmp(q1.mResults[0][i],q2.mResults[0][i]) != 0)
            return 1;
    for (i = 1; i < q1.mRows; i++)
        for (j = 0; j < q1.mColumns; j++)
        if (!(q1.mResults[i][j] == q2.mResults[i][j] || stricmp(q1.mResults[i][j],q2.mResults[i][j]) == 0))
            return 2;
    return 3;
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_GURU_DONE
 */

void Galaxql::OnGuruDoneClick( wxCommandEvent& event )
{
    GalaxqlApp * app = (GalaxqlApp*)wxTheApp;  

    if (mChapterSelect->IsEnabled())
    {
        mChapterSelect->Enable(0);
        int success = 0;
        switch (mChapterSelect->GetSelection()+1)
        {
        case 1:
            mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru01ok.htm");
            success=1;
            break;
        case 2:
            if (atoi(mUserQuery.mResults[1][0]) == 60*60*24)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru02ok.htm");
                success = 1;
            }
            else
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru02no.htm");
            }
            break;
        case 3:
            if (mUserQuery.mColumns > 3)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru03no1.htm");
            }
            else
            if (mGfxPanel->mStars == 0)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru_nostars.htm");
                success = -1;
            }
            else
            if (mUserQuery.mRows == 0)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru_noresults.htm");
            }
            else
            if (stricmp(mUserQuery.mResults[0][0],"x") == 0 &&
                stricmp(mUserQuery.mResults[0][1],"y") == 0 &&
                stricmp(mUserQuery.mResults[0][2],"z") == 0)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru03ok.htm");
                success = 1;
            }
            else
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru_notright.htm");
            }
            break;
        case 4:
            if (mUserQuery.mColumns != 4 ||
                !(stricmp(mUserQuery.mResults[0][0],"starid") == 0 &&
                  stricmp(mUserQuery.mResults[0][1],"x") == 0 &&
                  stricmp(mUserQuery.mResults[0][2],"y") == 0 &&
                  stricmp(mUserQuery.mResults[0][3],"z") == 0))
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru_wrongcolumns.htm");
            }
            else
            if (mGfxPanel->mStars == 0)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru_nostars.htm");
                success = -1;
            }
            else
            if (mUserQuery.mRows == 0)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru_noresults.htm");
            }
            else
            if (mUserQuery.mRows < 5)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru_notright.htm");
            }
            else
            if (min_col(mUserQuery, 1) < 0)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru04no2.htm");
            }
            else
            if (max_col(mUserQuery, 0) > 100)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru04no3.htm");
            }
            else
            if (!sorted_col(mUserQuery, 2))
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru04no4.htm");
            }
            else
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru04ok.htm");
                success = 1;
            }
            break;
        case 5:
            mGuruQuery.mColumns=mGuruQuery.mRows=0;
            sqlite3_exec(
                app->db,
                "SELECT SUM(y/x) FROM stars",
                sql_callback,
                &mGuruQuery,
                NULL);

            if (mGfxPanel->mStars == 0)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru_nostars.htm");
                success = -1;
            }
            else
            if (atof(mGuruQuery.mResults[1][0]) != atof(mUserQuery.mResults[1][0]))
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru05no.htm");
            }
            else
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru05ok.htm");
                success = 1;
            }            
            break;
        case 6:
            mGuruQuery.mColumns=mGuruQuery.mRows=0;
            sqlite3_exec(
                app->db,
                "SELECT s.starid,s.class "
                "FROM stars AS s, hilight AS h "
                "WHERE h.starid=s.starid",
                sql_callback,
                &mGuruQuery,
                NULL);

            if (mGfxPanel->mStars == 0)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru_nostars.htm");
                success = -1;
            }
            else
            if (mGuruQuery.mRows == 0)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru06no1.htm"); // no rows
            }
            else
            if (!equals_col(mGuruQuery,1,7))
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru06no2.htm"); // wrong star class 
            }
            else
            if (min_col(mGuruQuery,0) < 5000 || max_col(mGuruQuery,0) > 15000)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru06no3.htm"); // wrong starids
            }
            else
            if (mGuruQuery.mRows != 6)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru06no4.htm"); // too many/few results
            }
            else
            {
                success = 1;
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru06ok.htm");
            }
            break;
        case 7:
            mGuruQuery.mColumns=mGuruQuery.mRows=0;
            sqlite3_exec(
                app->db,
                "SELECT * from hilight",
                sql_callback,
                &mGuruQuery,
                NULL);

            if (mGfxPanel->mStars == 0)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru_nostars.htm");
                success = -1;
            }
            else
            if (mGuruQuery.mRows == 0)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru06no1.htm"); // no rows
            }
            else
            if (min_col(mGuruQuery,0) < 10000 || max_col(mGuruQuery,0) > 11000)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru07no.htm"); // outofrange
            }
            else
            if (mGuruQuery.mRows < 900)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru_toofewstars.htm"); // not enough stars
            }
            else
            {
                success = 1;
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru07ok.htm"); 
            }
            break;
        case 8:
            mGuruQuery.mColumns=mGuruQuery.mRows=0;
            sqlite3_exec(
                app->db,
                "SELECT MIN(starid) FROM stars",
                sql_callback,
                &mGuruQuery,
                NULL);

            if (mGfxPanel->mStars < 100)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru08no1.htm"); // too few stars
                success = -1;
            }
            else
            if (atoi(mGuruQuery.mResults[1][0]) < 10000)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru08no2.htm"); // starids less than 10k
            }
            else
            {
                mGuruQuery.mColumns=mGuruQuery.mRows=0;
                sqlite3_exec(
                    app->db,
                    "ROLLBACK;SELECT min(starid) from stars",
                    sql_callback,
                    &mGuruQuery,
                    NULL);
                // refresh starmap
                delete[] mGfxPanel->mVtx;
                delete[] mGfxPanel->mCol;
                mGfxPanel->mVtx = NULL;
                mGfxPanel->mCol = NULL;

                if (atoi(mGuruQuery.mResults[1][0]) > 9000)
                {
                    mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru08no3.htm"); // rollback failed
                    success = -1;
                }
                else
                {
                    mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru08ok.htm"); 
                    success=1;
                }
            }
            break;
        case 9:
            mGuruQuery.mColumns=mGuruQuery.mRows=0;
            sqlite3_exec(
                app->db,
                "SELECT z1/z2 FROM "
                "(SELECT AVG(z) AS z1 FROM stars WHERE starid > 10000 AND starid < 15000),"
                "(SELECT AVG(z) AS z2 FROM stars WHERE starid < 10000 OR starid > 15000)",
                sql_callback,
                &mGuruQuery,
                NULL);
            if (mGfxPanel->mStars < 20000)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru_toofewstars.htm"); // too few stars
                success = -1;
            }
            else
            if (fabs(atof(mGuruQuery.mResults[1][0]))>0.4)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru_notright.htm");
            }
            else
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru09ok.htm");
                success=1;
            }
            break;
        case 10:
            mGuruQuery.mColumns=mGuruQuery.mRows=0;
            sqlite3_exec(
                app->db,
                "SELECT DISTINCT stars.starid "
                "FROM stars, planets, moons "
                "WHERE stars.starid = planets.starid "
                "AND planets.planetid = moons.planetid "
                "AND moons.orbitdistance > 3000 "
                "AND stars.starid > 20000 "
                "UNION ALL "
                "SELECT * FROM hilight "
                "ORDER BY starid",
                sql_callback,
                &mGuruQuery,
                NULL);
            if (mGfxPanel->mStars < 20000)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru_toofewstars.htm"); // too few stars
                success = -1;
            }
            else
            if ((mGuruQuery.mRows & 1) == 0 || !check_pairs(mGuruQuery))
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru10no.htm"); // no pairs
            }
            else
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru10ok.htm");
                success=1;
            }
            break;            
        case 11:
            mGuruQuery.mColumns=mGuruQuery.mRows=0;
            sqlite3_exec(
                app->db,
                "SELECT stars.starid FROM stars, planets, "
                "(SELECT MAX(orbitdistance) AS os FROM planets) "
                "WHERE stars.starid=planets.starid AND planets.orbitdistance=os "
                "UNION ALL "
                "SELECT * FROM hilight "
                "ORDER BY starid",
                sql_callback,
                &mGuruQuery,
                NULL);
            if (mGfxPanel->mStars < 20000)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru_toofewstars.htm"); // too few stars
                success = -1;
            }
            else
            if ((mGuruQuery.mRows & 1) == 0 || !check_pairs(mGuruQuery))
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru11no.htm"); // no pairs
            }
            else
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru11ok.htm");
                success=1;
            }
            break;            
        case 12:
            mGuruQuery.mColumns=mGuruQuery.mRows=0;
            sqlite3_exec(
                app->db,
                "SELECT stars.name AS starname, "
                "((class+7)*intensity)*1000000 AS startemp, "
                "planets.name AS planetname, "
                "((class+7)*intensity)*1000000-orbitdistance*50 AS planettemp "
                "FROM stars "
                "LEFT OUTER JOIN planets "
                "ON stars.starid=planets.starid "
                "WHERE stars.starid < 100",
                sql_callback,
                &mGuruQuery,
                NULL);
            if (mGfxPanel->mStars < 20000)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru_toofewstars.htm"); // too few stars
                success = -1;
            }
            else
            if (mUserQuery.mColumns != 4)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru12no1.htm"); // wrong number of columns
            }
            else
            if (mUserQuery.mRows != mGuruQuery.mRows)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru12no2.htm"); // wrong number of rows
            }
            else
            {
                int c = compare_queries(mGuruQuery, mUserQuery);
                if (c == 1)
                {
                    mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru12no3.htm"); // wrong column names
                }
                else
                if (c == 2)
                {
                    mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru12no4.htm"); // wrong calc?
                }
                else
                {
                    mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru12ok.htm");
                    success=1;
                }
            }
            break;        
        case 13:
            mGuruQuery.mColumns=mGuruQuery.mRows=0;
            sqlite3_exec(
                app->db,
                "SELECT * FROM numbers "
                "UNION ALL "
                "SELECT 3 AS three, intensity, x "
                "FROM stars "
                "ORDER BY x",
                sql_callback,
                &mGuruQuery,
                NULL);
            if (mGfxPanel->mStars < 20000)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru_toofewstars.htm"); // too few stars
                success = -1;
            }
            else
            if (mGuruQuery.mRows == 0)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru13no3.htm"); // no view
            }
            else
            if (mGuruQuery.mColumns > 3)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru13no1.htm"); // extra columns
            }
            else
            if (!check_pairs(mGuruQuery))
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru13no2.htm"); // something else went wrong
            }
            else
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru13ok.htm");
                success=1;
            }
            break;        
        case 14:
            mGuruQuery.mColumns=mGuruQuery.mRows=0;
            sqlite3_exec(
                app->db,
                "SELECT * FROM colors ORDER BY color",
                sql_callback,
                &mGuruQuery,
                NULL);
            if (mGuruQuery.mRows == 0)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru14no1.htm"); // no table / wrong columns
            }
            else
            if (mGuruQuery.mColumns > 2)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru14no2.htm"); // extra columns
            }
            else
            if (mGuruQuery.mColumns != 2 ||
                stricmp(mGuruQuery.mResults[0][0],"color") != 0 ||
                stricmp(mGuruQuery.mResults[0][1],"description") != 0)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru14no3.htm"); // wrong columns
            }
            else
            if (mGuruQuery.mRows != 15 || 
                min_col(mGuruQuery,0) != -3 || 
                max_col(mGuruQuery,0) != 10)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru14no4.htm"); // wrong number of rows
            }
            else // TODO: smarter checks ('color' values)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru14ok.htm");
                success=1;
            }
            break;        
        case 15:
            mGuruQuery.mColumns=mGuruQuery.mRows=0;
            sqlite3_exec(
                app->db,
                "SELECT *,id,quote FROM quotes",
                sql_callback,
                &mGuruQuery,
                NULL);
            if (mGuruQuery.mRows == 0)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru15no1.htm"); // no table / wrong columns
            }
            else
            if (mGuruQuery.mColumns != 4)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru15no2.htm"); // extra columns
            }
            else
            if (stricmp(mGuruQuery.mResults[0][0],"id") != 0 ||
                stricmp(mGuruQuery.mResults[0][1],"quote") != 0)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru15no3.htm"); // wrong columns
            }
            else
            {
                mGuruQuery.mColumns=mGuruQuery.mRows=0;
                sqlite3_exec(
                    app->db,
                    "DELETE FROM quotes WHERE id=7357",
                    sql_callback,
                    &mGuruQuery,
                    NULL);
                mGuruQuery.mColumns=mGuruQuery.mRows=0;
                sqlite3_exec(
                    app->db,
                    "INSERT INTO quotes (id, quote) VALUES (7357, NULL)",
                    sql_callback,
                    &mGuruQuery,
                    NULL);
                mGuruQuery.mColumns=mGuruQuery.mRows=0;
                sqlite3_exec(
                    app->db,
                    "SELECT * FROM quotes WHERE id=7357",
                    sql_callback,
                    &mGuruQuery,
                    NULL);
                if (mGuruQuery.mRows != 0)
                {
                    mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru15no4.htm"); // could insert NULL to non-null column
                }
                else 
                {
                    mGuruQuery.mColumns=mGuruQuery.mRows=0;
                    sqlite3_exec(
                        app->db,
                        "INSERT INTO quotes (id, quote) VALUES (7357, \"test1\")",
                        sql_callback,
                        &mGuruQuery,
                        NULL);
                    mGuruQuery.mColumns=mGuruQuery.mRows=0;
                    sqlite3_exec(
                        app->db,
                        "INSERT INTO quotes (id, quote) VALUES (7357, \"test2\")",
                        sql_callback,
                        &mGuruQuery,
                        NULL);
                    mGuruQuery.mColumns=mGuruQuery.mRows=0;
                    sqlite3_exec(
                        app->db,
                        "SELECT * FROM quotes WHERE id=7357",
                        sql_callback,
                        &mGuruQuery,
                        NULL);

                    if (mGuruQuery.mRows != 2)
                    {
                        mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru15no5.htm"); // could insert to same id twice
                    }
                    else
                    {
                        mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru15ok.htm");
                        success=1;
                    }
                }
            }
            break;        
        case 16:
            mGuruQuery.mColumns=mGuruQuery.mRows=0;
            sqlite3_exec(
                app->db,
                "SELECT moredata FROM my_table UNION ALL SELECT COUNT(moredata) FROM my_table",
                sql_callback,
                &mGuruQuery,
                NULL);
            if (mGuruQuery.mRows == 0)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru16no1.htm"); // no table / wrong columns
            }
            else
            if (mGuruQuery.mRows < 5)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru16no2.htm"); // too little data
            }
            else
            if (atoi(mGuruQuery.mResults[mGuruQuery.mRows-1][0]) < mGuruQuery.mRows-4)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru16no3.htm"); // too many NULLs
            }
            else
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru16ok.htm");
                success=1;
            }
            break;        
        case 17:
            mGuruQuery.mColumns=mGuruQuery.mRows=0;
            sqlite3_exec(
                app->db,
                "SELECT starid, orbitals "
                "FROM (SELECT stars.starid AS starid, "
                " COUNT(planets.planetid)+COUNT(moons.moonid) AS orbitals "
                " FROM planets, stars, moons "
                " WHERE stars.starid=planets.starid "
                " AND moons.planetid=planets.planetid "
                " GROUP BY stars.starid "
                " ORDER BY orbitals DESC "
                " LIMIT 1) "
                "UNION ALL "
                "SELECT starid,orbitals "
                "FROM (SELECT stars.starid AS starid, "
                " COUNT(planets.planetid)+COUNT(moons.moonid) AS orbitals "
                " FROM planets, stars, moons, "
                "  (SELECT starid AS hilited "
                "  FROM hilight "
                "  ORDER BY starid "
                "  LIMIT 1) "
                " WHERE stars.starid=hilited "
                " AND stars.starid=planets.starid "
                " AND moons.planetid=planets.planetid "
                " GROUP BY stars.starid "
                " ORDER BY orbitals DESC "
                " LIMIT 1)",
                sql_callback,
                &mGuruQuery,
                NULL);
            if (mGuruQuery.mRows != 3 || atoi(mGuruQuery.mResults[1][1]) != atoi(mGuruQuery.mResults[2][1]))
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru17no.htm"); // wrong star
            }
            else
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru17ok.htm");
                success=1;
            }
            break;        
        case 18:
            mGuruQuery.mColumns=mGuruQuery.mRows=0;
            sqlite3_exec(
                app->db,
                "SELECT starid FROM planets "
                "EXCEPT "
                "SELECT starid*2 AS starid FROM planets "
                "INTERSECT "
                "SELECT starid*3 AS starid FROM planets ",
                sql_callback,
                &mGuruQuery,
                NULL);
            if (mGfxPanel->mStars < 20000)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru_toofewstars.htm"); // too few stars
                success = -1;
            }
            else
            {
                int c = compare_queries(mUserQuery, mGuruQuery);
                switch (c)
                {
                case -2:
                    mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru18no1.htm"); // wrong number of columns
                    break;
                case -1:
                    mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru18no2.htm"); // wrong number of rows
                    break;
                case 0:
                    mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru_noresults.htm"); // no rows
                    break;
                case 1:
                    mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru_wrongcolumns.htm"); // wrong column names
                    break;
                case 2:
                    mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru18no3.htm"); // data mismatch
                    break;
                default:
                    mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru18ok.htm");
                    success=1;                
                }
            }
            break;        
        case 19:
            mGuruQuery.mColumns=mGuruQuery.mRows=0;
            sqlite3_exec(
                app->db,
                "SELECT MAX(starid) FROM stars",
                sql_callback,
                &mGuruQuery,
                NULL);
            if (mGfxPanel->mStars < 20000)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru_toofewstars.htm"); // too few stars
                success = -1;
            }
            else
            {
                int maxid = atoi(mGuruQuery.mResults[1][0]);
                char temp[256];
                sprintf(temp, "INSERT INTO hilight (starid) VALUES (%d)", maxid);
                mGuruQuery.mColumns=mGuruQuery.mRows=0;
                sqlite3_exec(
                    app->db,
                    temp,
                    sql_callback,
                    &mGuruQuery,
                    NULL);
                maxid++;
                sprintf(temp, "INSERT INTO stars (starid,x,y,z,class,intensity) VALUES (%d,5,4,3,2,1)", maxid);
                mGuruQuery.mColumns=mGuruQuery.mRows=0;
                sqlite3_exec(
                    app->db,
                    temp,
                    sql_callback,
                    &mGuruQuery,
                    NULL);
                mGuruQuery.mColumns=mGuruQuery.mRows=0;
                sqlite3_exec(
                    app->db,
                    "SELECT starid FROM hilight",
                    sql_callback,
                    &mGuruQuery,
                    NULL);

                if (mGuruQuery.mRows != 2 || atoi(mGuruQuery.mResults[1][0]) != maxid)
                {
                    mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru19no.htm"); // no view
                }
                else
                {
                    mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru19ok.htm");
                    success=1;
                }
            }
            break;        
        case 20:
            if (mGfxPanel->mGateways == 0)
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru20no.htm"); // no gateways
            }
            else
            {
                mGuruSpeaks->LoadPage("file:galaxql.dat#zip:guru20ok.htm");
                success=1;
            }
            break;        

        default:
            success=2;
        }
        if (success>0)
        {
            if (success == 1)
                mGuruPicture->SetBitmap(mGuruHappy);
            else
                mGuruPicture->SetBitmap(mGuruEaster);
            mChapterSelect->SetSelection(mChapterSelect->GetSelection()+1);
            mGuruDone->SetLabel("Ok!");
        }
        else
        {
            if (success == 0)
                mGuruPicture->SetBitmap(mGuruSad);
            else
                mGuruPicture->SetBitmap(mGuruBadDb);
            mGuruDone->SetLabel("I'll try again..");
        }
    }
    else
    {
        mGuruPicture->SetBitmap(mGuruNormal);
        mChapterSelect->Enable(1);
        mGuruDone->SetLabel("Ok, I'm done..");
        OnChapterselectSelected(event);
    }
    event.Skip();
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_LOADQUERY
 */

void Galaxql::OnLoadqueryClick( wxCommandEvent& event )
{
    wxFileDialog fd(this, 
        wxT("Load query"), 
        wxT(""), 
        wxT("query.txt"), 
        wxT("txt files (*.txt)|*.txt"), 
        wxOPEN);
    if (fd.ShowModal() == wxID_OK)
    {
        FILE * f;
        wxString bah;
        bah = fd.GetDirectory() + "/" + fd.GetFilename();
        f = fopen(bah,"rb");
        fseek(f,0,SEEK_END);
        int len = ftell(f);
        fseek(f,0,SEEK_SET);
        char * d = new char[len+1];
        fread(d,len,1,f);
        fclose(f);
        d[len]=0;
        mQuery->SetValue(d);
        delete[] d;
    }
    event.Skip();
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_SAVEQUERY
 */

void Galaxql::OnSavequeryClick( wxCommandEvent& event )
{
    wxFileDialog fd(this, 
        wxT("Save query"), 
        wxT(""), 
        wxT("query.txt"), 
        wxT("txt files (*.txt)|*.txt"), 
        wxSAVE | wxOVERWRITE_PROMPT);
    if (fd.ShowModal() == wxID_OK)
    {
        FILE * f;
        wxString bah;
        bah = fd.GetDirectory() + "/" + fd.GetFilename();
        f = fopen(bah,"wb");
        wxString s = mQuery->GetValue();
        const char *d = s;
        fwrite(d,s.Len(),1,f);
        fclose(f);
    }
    event.Skip();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_MENURUNQUERY
 */

void Galaxql::OnMenurunqueryClick( wxCommandEvent& event )
{
    OnRunqueryClick(event);
    event.Skip();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_MENUSAVEQUERY
 */

void Galaxql::OnMenusavequeryClick( wxCommandEvent& event )
{
    OnSavequeryClick( event );
}

/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_MENULOADQUERY
 */

void Galaxql::OnMenuloadqueryClick( wxCommandEvent& event )
{
    OnLoadqueryClick( event );
}


/*!
 * wxEVT_COMMAND_CHOICE_SELECTED event handler for ID_CHAPTERSELECT
 */

void Galaxql::OnChapterselectSelected( wxCommandEvent& event )
{
    int selection = mChapterSelect->GetSelection()+1;
    char temp[256];
    if (selection < 21)
        sprintf(temp,"file:galaxql.dat#zip:guru%02d.htm",selection);
    else
        sprintf(temp,"file:galaxql.dat#zip:guruend.htm",selection);

    mGuruSpeaks->LoadPage(temp);

    event.Skip();
}


/*!
 * wxEVT_COMMAND_NOTEBOOK_PAGE_CHANGED event handler for ID_NOTEBOOK
 */

void Galaxql::OnNotebookPageChanged( wxNotebookEvent& event )
{
    if (mQuery && event.GetSelection()==1)
        mQuery->SetFocus();
    if (mGuruSpeaks && event.GetSelection()==0)
        mGuruSpeaks->SetFocus();
    event.Skip();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_MENUGLOW
 */

void Galaxql::OnMenuglowClick( wxCommandEvent& event )
{
    mGfxPanel->mRenderWithGlow = !mGfxPanel->mRenderWithGlow;
    wxMenuBar *mb = GetMenuBar();
    mb->Check(ID_MENUGLOW,mGfxPanel->mRenderWithGlow!=0);
    
    event.Skip();
}



/*!
 * wxGuruSpeaksPanel type definition
 */

IMPLEMENT_DYNAMIC_CLASS( wxGuruSpeaksPanel, wxHtmlWindow )

/*!
 * wxGuruSpeaksPanel event table definition
 */

BEGIN_EVENT_TABLE( wxGuruSpeaksPanel, wxHtmlWindow )

////@begin wxGuruSpeaksPanel event table entries
////@end wxGuruSpeaksPanel event table entries

END_EVENT_TABLE()

/*!
 * wxGuruSpeaksPanel constructors
 */

wxGuruSpeaksPanel::wxGuruSpeaksPanel( )
{
}

wxGuruSpeaksPanel::wxGuruSpeaksPanel( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
    Create(parent, id, pos, size, style);
}

/*!
 * wxGuruSpeaksPanel creator
 */

bool wxGuruSpeaksPanel::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin wxGuruSpeaksPanel member initialisation
////@end wxGuruSpeaksPanel member initialisation

////@begin wxGuruSpeaksPanel creation
    wxHtmlWindow::Create( parent, id, pos, size, style );

    CreateControls();
////@end wxGuruSpeaksPanel creation
    return true;
}

/*!
 * Control creation for wxGuruSpeaksPanel
 */

void wxGuruSpeaksPanel::CreateControls()
{    
////@begin wxGuruSpeaksPanel content construction
    // Generated by DialogBlocks, 12/04/05 22:08:22 (Personal Edition)

    wxGuruSpeaksPanel* itemHtmlWindow43 = this;

////@end wxGuruSpeaksPanel content construction
}

/*!
 * Should we show tooltips?
 */

bool wxGuruSpeaksPanel::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap wxGuruSpeaksPanel::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin wxGuruSpeaksPanel bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end wxGuruSpeaksPanel bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon wxGuruSpeaksPanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin wxGuruSpeaksPanel icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end wxGuruSpeaksPanel icon retrieval
}

void wxGuruSpeaksPanel::OnLinkClicked(const wxHtmlLinkInfo& link)
{
    GalaxqlApp * app = (GalaxqlApp*)wxTheApp;  
    Galaxql * g = (Galaxql*)app->GetTopWindow();
    g->mQuery->SetValue(link.GetHref());
    g->mNotebook->SetSelection(1);
    g->mQuery->SetFocus();
    
}
